using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using NLog;
using System.IO;
using System.Collections;
using System.Data.SqlClient;

namespace IdcgLibrary
{
    [Serializable]
    public class IdLibrary
    {
        #region Constants

        public static string CLASS_NAME = typeof(IdLibrary).ToString();

        // Effectively a constant
        private static Logger logger = LogManager.GetCurrentClassLogger();

        #endregion

        #region Library Constants

        // Result code for 3 possible results
        public enum TriResult { SUCCESS, NOTFOUND, FAIL };

        /// <summary>
        /// How much detail to do LogitDetail at
        /// </summary>
        public enum LogDocDepth { MAX, FULL, MODERATE, MINIMUM };

        // Default names for key database values
        public const string DEFT_DBNAME = "OfficeDemo";
        public const string DEFT_USERID = "OfficeDemoUser";
        public const string DEFT_USERPASSWORD = "officeDemo";

        #endregion

        #region Library Routines

        /// <summary>
        /// Return true on NOTFOUND or ERROR.
        /// </summary>
        /// <param name="aTableName">Table name to drop</param>
        public static bool AnyIdError(TriResult aResult)
        {
            if (aResult != TriResult.SUCCESS)
                return true;
            else
                return false;
        }

        #region Database Support

        /// <summary>
        /// Convert to SQL server equivalent BIT field. Return 0 for false, 1 for true.
        /// </summary>
        /// <param name="aValue">boolean value to convert</param>
        /// <returns></returns>
        public static string BoolToBit(bool aValue)
        {
            if (aValue == true) return "1";
            else return "0";
        }

        /// <summary>
        /// Return SqlConnection on success, null on fail.
        /// </summary>
        /// <param name="aSourceName">SQL server to use or null for default</param>
        /// <param name="aDbName">Database on server to connect to</param>
        /// <returns></returns>
        public static SqlConnection OpenDbConnection(string aSourceName, string aDbName, string aUserID, string aPassword)
        {
            try
            {
                logger.Info("OpenDbConnection:1: aSourceName='"+aSourceName+"', aDbName='"+aDbName+"'");

                SqlConnectionStringBuilder bldr = new SqlConnectionStringBuilder();
                SqlConnection myConnection;

                if (aSourceName == null)
                    bldr.DataSource = "localhost\\SQLExpress"; // Sql server to use
                else
                    bldr.DataSource = aSourceName; // Sql server to use

                bldr.InitialCatalog = aDbName;     //The database on the server that you want to connect to.

                if (aUserID == null)
                    bldr.UserID = "OfficeDemoUser"; //The user id
                else
                    bldr.UserID = aUserID; //The user id

                if (aUserID == null)
                    bldr.Password = "officeDemo";  //The pwd for said user account
                else
                    bldr.Password = aPassword;

                myConnection = new SqlConnection(bldr.ConnectionString);

                // Open the connection.
                myConnection.Open();
                return myConnection;
            }
            catch (Exception ex)
            {
                logger.Error("OpenDbConnection: Got ex: "+ex);
                return null;
            }
        }

        /// <summary>
        /// Run the given SQL command.
        /// Return SUCCESS, FAIL, or NOTFOUND
        /// </summary>
        /// <param name="aCommand">Command to run</param>
        /// <param name="aDbName">Database name to connect to</param>
        /// <param name="aUserID">User ID to connect to database</param>
        /// <param name="aPassword">Password to connect to database</param>
        public static TriResult RunSqlNonQuery(string aCommand, string aDbName, string aUserID, string aPassword)
        {
            return RunSqlNonQuery(aCommand, null, null, aDbName, aUserID, aPassword);
        }

        /// <summary>
        /// Run the given SQL command, handling connection start/stop and transaction and passwords.
        /// Return SUCCESS, FAIL, or NOTFOUND
        /// </summary>
        /// <param name="aCommand">Command to run</param>
        public static TriResult RunSqlNonQuery(string aCommand)
        {
            return RunSqlNonQuery(aCommand, null, null, DEFT_DBNAME, DEFT_USERID, DEFT_USERPASSWORD);
        }

        /// <summary>
        /// Run the given SQL command, handling transaction and passwords.
        /// Return SUCCESS, FAIL, or NOTFOUND
        /// </summary>
        /// <param name="aCommand">Command to run</param>
        /// <param name="aConnection">Connection to use, or null for own open/close</param>
        public static TriResult RunSqlNonQuery(string aCommand, SqlConnection aConnection)
        {
            if (aConnection == null)
            {
                logger.Error("RunSqlNonQuery: Null connection not allowed");
                return TriResult.FAIL;
            }

            return RunSqlNonQuery(aCommand, aConnection, null, DEFT_DBNAME, DEFT_USERID, DEFT_USERPASSWORD);
        }

        /// <summary>
        /// Run the given SQL command.
        /// Return SUCCESS, FAIL, or NOTFOUND
        /// </summary>
        /// <param name="aCommand">Command to run</param>
        /// <param name="aConnection">Connection to use, or null for own open/close</param>
        public static TriResult RunSqlNonQuery(string aCommand, SqlConnection aConnection, SqlTransaction aTrans)
        {
            if (aConnection == null)
            {
                logger.Error("RunSqlNonQuery: Null connection not allowed");
                return TriResult.FAIL;
            }

            if (aTrans == null)
            {
                logger.Error("RunSqlNonQuery: Null transaction not allowed");
                return TriResult.FAIL;
            }

            return RunSqlNonQuery(aCommand, aConnection, aTrans, DEFT_DBNAME, DEFT_USERID, DEFT_USERPASSWORD);
        }

        /// <summary>
        /// Run the given SQL command. This should only be called by ICDG routines!
        /// Return SUCCESS, FAIL, or NOTFOUND
        /// </summary>
        /// <param name="aCommand">Command to run</param>
        /// <param name="aConnection">Connection to use, or null for own open/close</param>
        public static TriResult RunSqlNonQuery(string aCommand, SqlConnection aConnection, SqlTransaction aTrans, string aDbName, string aUserID, string aPassword)
        {
            SqlConnection myConnection = null;
            int recCount;
            SqlCommand myCommand;

            try
            {
                logger.Trace("RunSqlNonQuery:1: aCommand='"+aCommand+"'");

                if (aConnection != null)
                    myConnection = aConnection;
                else
                    myConnection = OpenDbConnection(null, aDbName, aUserID, aPassword);

                if (myConnection == null)
                    return TriResult.FAIL;

                if (aTrans != null)
                    myCommand = new SqlCommand(aCommand, myConnection, aTrans);
                else
                    myCommand = new SqlCommand(aCommand, myConnection);

                recCount = myCommand.ExecuteNonQuery();

                if (aConnection == null)
                    myConnection.Close();

                logger.Info("RunSqlNonQuery:3: aCommand='"+aCommand+"', returned cnt="+recCount);
                return TriResult.SUCCESS;
            }
            catch (Exception ex)
            {
                TriResult rc;

                // Real error, put in error log
                if (ex.Message.Contains("because it does not exist") == false)
                {
                    logger.Error("RunSqlNonQuery: Got ex: "+ex.Message+", returning FAIL");
                    rc = TriResult.FAIL;
                }
                else   // Table/rec not found error
                {
                    logger.Info("RunSqlNonQuery: Got ex: "+ex.Message+", returning NOTFOUND");
                    rc = TriResult.NOTFOUND;
                }
                logger.Info("RunSqlNonQuery: aCommand was'"+aCommand+"'");

                if (aConnection == null && myConnection != null)
                    myConnection.Close();
                return rc;
            }
        }

        #endregion

        #region Support

        /// <summary>
        /// Return true if we should try to log, false if not.
        /// </summary>
        /// <param name="aLogger"></param>
        /// <param name="aLevel"></param>
        /// <returns></returns>
        public static bool WithinLoggerLevel(Logger aLogger, LogLevel aLevel)
        {
            if (aLevel == LogLevel.Trace && aLogger.IsTraceEnabled == true)
                return true;

            else if (aLevel == LogLevel.Debug && aLogger.IsDebugEnabled == true)
                return true;

            else if (aLevel == LogLevel.Info && aLogger.IsInfoEnabled == true)
                return true;

            else if (aLevel == LogLevel.Warn && aLogger.IsWarnEnabled == true)
                return true;

            else if (aLevel == LogLevel.Error)
                return true;

            else
                return false;
        }

        #endregion

        #endregion

        #region Common Routines

        // Force utf-8 instead of 16
        public class Utf8StringWriter : StringWriter
        {
            public override Encoding Encoding
            {
                get { return Encoding.UTF8; }
            }
        }

        #endregion

    }
}
